
#include <math.h>
#include <string.h>
#include <limits.h>

#include "fasterac/sampler.h"
#include "fasterac/sampling.h"

//  ylsb = 0.004558563 mV    (2*2390 / 2^20)
#define TO_MV 0.004558563
#define TO_NS 2.0
#define NOVAL -1000000.0

//----------  local tools  ---------------------------------//


   double sampling_xpoint_ns_order1 (sampling_t const* s,          //  LINEAR INPERPOLATION
                                     double            xval_mV,
                                     int               rising_edge,
                                     double            from_ns,
                                     double            to_ns);

   double sampling_xpoint_ns_order2 (sampling_t const* s,          //  POLY 2
                                     double            xval_mV,
                                     int               rising_edge,
                                     double            from_ns,
                                     double            to_ns);

   double i2ns (double first_ns, unsigned short i) {
      return first_ns + i * TO_NS;
   }

   unsigned short ns2i (double first_ns, double ns) {
      return round ((ns - first_ns) / TO_NS);
   }

   double raw2mV (sampler_range_t range,
                  short           rawdata) {
      return rawdata * pow (2, range) * TO_MV;
   }


   double value (sampling_t const* s,
                 unsigned int      i) {
      return raw2mV (s->range_mV, s->data [i]);
   }

//----------------------------------------------------------//

   int sampling_init (sampling_t*     s,        //  return 0 when ok
                      sampler_meta_t  meta,     //  sampler meta data (label, time shift & voltage range)
                      faster_data_p   sampler)  //  sampler as faster data
   {
      unsigned short  label        = meta.label;
      double          before_th_ns = TO_NS * round (meta.shift_ns / TO_NS);
      sampler_range_t range_mV     = meta.range_mV;
      if ((before_th_ns < 0.0) || (before_th_ns >= SAMPLER_NB_PTS_MAX * TO_NS))
      {
         return 1;  //  error
      } else {
         s->range_mV    = range_mV;
         s->label       = label;
         s->ts_ns       = faster_data_clock_ns  (sampler);
         s->nb_pts      = faster_data_load_size (sampler) / 2;
         s->first_ns    = -1.0 * before_th_ns;
         s->last_ns     = i2ns (s->first_ns, s->nb_pts - 1);
         memcpy (s->data, faster_data_load_p (sampler), 2 * s->nb_pts);
         //  max, min, qtot
         {
            double val;
            double qtot = 0.0;
            double max  = INT_MIN;
            double min  = INT_MAX;
            int    maxi = 0;
            int    mini = 0;
            int    i;
            for (i = 0; i < s->nb_pts; i++) {
               val = value (s, i);
               qtot += val * TO_NS;
               if (val > max) {
                  maxi = i;
                  max  = val;
               }
               if (val < min) {
                  mini = i;
                  min  = val;
               }
            }
            s->max_mV    = max;
            s->min_mV    = min;
            s->max_ns    = i2ns (s->first_ns, maxi);
            s->min_ns    = i2ns (s->first_ns, mini);
            s->qtot_mVns = qtot;
         }
         return 0; // Ok
      }
   }


   unsigned short sampling_label (sampling_t const* s) {
      return s->label;
   }


   //long double sampling_clock_ns (sampling_t const* s) {
   double sampling_clock_ns (sampling_t const* s) {
      return s->ts_ns;
   }


   unsigned short sampling_nb_pts (sampling_t const* s) {
      return s->nb_pts;
   }


   double sampling_first_ns (sampling_t const* s) {
      return s->first_ns;
   }


   double sampling_last_ns (sampling_t const* s) {
      return s->last_ns;
   }


   double sampling_value_mV (sampling_t const* s,
                             double            at_ns) {
      return value (s, ns2i (s->first_ns, at_ns));
   }


   double sampling_max_ns (sampling_t const* s,
                           double            from_ns,
                           double            to_ns) {
      int    ifrom = ns2i (s->first_ns, from_ns);
      int    ito   = ns2i (s->first_ns, to_ns) + 1;
      int    i     = 0;
      int    imax  = 0;
      double val   = 0.0;
      double max   = INT_MIN;
      for (i = ifrom; i < ito; i++) {
         val = value (s, i);
         if (val > max) {
            max  = val;
            imax = i;
         }
      }
      return sampling_to_ns (s, i);
   }


   double sampling_max_mV (sampling_t const* s,
                           double            from_ns,
                           double            to_ns) {
      int    ifrom = ns2i (s->first_ns, from_ns);
      int    ito   = ns2i (s->first_ns, to_ns) + 1;
      int    i     = 0;
      double val   = 0.0;
      double max   = INT_MIN;
      for (i = ifrom; i < ito; i++) {
         val = value (s, i);
         if (val > max) {
            max  = val;
         }
      }
      return max;
   }


   double sampling_min_ns (sampling_t const* s,
                           double            from_ns,
                           double            to_ns) {
      int    ifrom = ns2i (s->first_ns, from_ns);
      int    ito   = ns2i (s->first_ns, to_ns) + 1;
      int    i     = 0;
      int    imin  = 0;
      double val   = 0.0;
      double min   = INT_MAX;
      for (i = ifrom; i < ito; i++) {
         val = value (s, i);
         if (val < min) {
            min  = val;
            imin = i;
         }
      }
      return sampling_to_ns (s, i);
   }


   double sampling_min_mV (sampling_t const* s,
                           double            from_ns,
                           double            to_ns) {
      int    ifrom = ns2i (s->first_ns, from_ns);
      int    ito   = ns2i (s->first_ns, to_ns) + 1;
      int    i     = 0;
      double val   = 0.0;
      double min   = INT_MAX;
      for (i = ifrom; i < ito; i++) {
         val = value (s, i);
         if (val < min) {
            min  = val;
         }
      }
      return min;
   }


   double sampling_average_mV (sampling_t const* s,
                               double            from_ns,
                               double            to_ns) {
      int ifrom  = ns2i (s->first_ns, from_ns);
      int ito    = ns2i (s->first_ns, to_ns) + 1;
      int i      = 0;
      double avg = 0.0;
      for (i = ifrom; i < ito; i++) {
         avg += value (s, i);
      }
      return avg / (ito - ifrom);
   }


   double sampling_charge_mVns  (sampling_t const* s,
                                 double            from_ns,
                                 double            to_ns) {
      double q     = 0.0;
      int    ifrom = sampling_to_idx (s, from_ns);
      int    ito   = sampling_to_idx (s, to_ns) + 1;
      int    i;
      for (i = ifrom; i < ito; i++) {
         q += value (s, i) * TO_NS;
      }
      return q;
   }


   double sampling_xpoint_ns (sampling_t const* s,
                              double            xval_mV,
                              int               rising_edge,
                              double            from_ns,
                              double            to_ns,
                              int               interpol) {
      if (interpol == 1) {
         return sampling_xpoint_ns_order1 (s, xval_mV, rising_edge, from_ns, to_ns);
      } else if (interpol == 2) {
         return sampling_xpoint_ns_order2 (s, xval_mV, rising_edge, from_ns, to_ns);
      } else {
         printf ("sampling_xpoint_ns ERROR order not supported\n");
         return to_ns + 1.0;
      }
   }


   double sampling_xpoint_ns_order1 (sampling_t const* s,          //  LINEAR INPERPOLATION
                                     double            xval_mV,
                                     int               rising_edge,
                                     double            from_ns,
                                     double            to_ns) {
      double t0, v0, v1;
      int    i, j, ok;
      int    i_from = sampling_to_idx (s, from_ns);
      int    i_to   = sampling_to_idx (s, to_ns) + 1;
      i  = i_from;
      v1 = value (s, i);
      ok = 0;
      if (rising_edge) {
         while (i < i_to) {
            v0 = v1;
            v1 = value (s, i + 1);
            if ((v0 < xval_mV) && (v1 >= xval_mV)) {
               ok = 1;
               break;
            }
            i++;
         }
      } else {
         while (i < i_to) {
            v0 = v1;
            v1 = value (s, i + 1);
            if ((v0 > xval_mV) && (v1 <= xval_mV)) {
               ok = 1;
               break;
            }
            i++;
         }
      }
      if (ok) {
         t0 = sampling_to_ns (s, i);
         //printf ("i=%d t0=%fns v0=%fmV xval=%fmV v1=%fmV ratio=%f\n", i, t0, v0, xval_mV, v1, (xval_mV - v0) / (v1 - v0));
         return t0 + TO_NS * (xval_mV - v0) / (v1 - v0);
      } else {
         return to_ns + 1.0;
      }
   }


   double sampling_xpoint_ns_order2 (sampling_t const* s,      //  INTERPOLATION POLY2
                                     double            xval_mV,
                                     int               rising_edge,
                                     double            from_ns,
                                     double            to_ns) {
      double inc = 2.0; //  ns
      double a, b, c;
      double C;
      double delta;
      double z1, z2;
      double t0, t1, v0, v1, v2;
      int    i, j, ok;
      int    i_from = sampling_to_idx (s, from_ns) - 1;
      int    i_to   = sampling_to_idx (s, to_ns)   + 1;   //  TODO i min i max !!!
      i  = i_from;
      v1 = value (s, i);
      i++;
      v2 = value (s, i);
      ok = 0;
      if (rising_edge) {
         while (!ok && (i < i_to)) {
            v0 = v1;
            v1 = v2;
            v2 = value (s, i + 1);
            if ((v1 < xval_mV) && (v2 >= xval_mV)) {
               ok = 1;
               break;
            }
            i++;
         }
      } else {
         while (!ok && (i < i_to)) {
            v0 = v1;
            v1 = v2;
            v2 = value (s, i + 1);
            if ((v1 > xval_mV) && (v2 <= xval_mV)) {
               ok = 1;
               break;
            }
            i++;
         }
      }
      if (ok) {
         t1 = sampling_to_ns (s, i);
         t0 = t1 - inc;
         a  = (v2 - 2*v1 + v0) / (2*inc*inc);
         b  = (v1 - v0) / inc - a*(t0 + t1);
         c  = v0 - a*t0*t0 - b*t0;
         C  = c - xval_mV;
         delta = b*b - 4*a*C;
         z1    = (-b - sqrt (delta)) / (2*a);
         z2    = (-b + sqrt (delta)) / (2*a);
         return z2;
      } else {
         return to_ns + 1.0;
      }
   }


   unsigned int sampling_to_idx (sampling_t const* s,
                                 double            at_ns) {
      return ns2i (s->first_ns, at_ns);
   }


   double sampling_to_ns (sampling_t const* s,
                          unsigned int      idx) {
      return i2ns (s->first_ns, idx);
   }


   double sampling_to_mV (sampling_t const* s,
                          short             rawdata) {
      return raw2mV (s->range_mV, rawdata);
   }


